@@ -852,10 +852,24 @@ static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
     RECT rect, rectParent;
     HWND parent, child;
     HRGN hrgn, tmp;
-    int xspacing, yspacing;
+    int x, y, xspacing, yspacing;
+    MINIMIZEDMETRICS metrics;
+
+    metrics.cbSize = sizeof(metrics);
+    SystemParametersInfoW( SPI_GETMINIMIZEDMETRICS, sizeof(metrics), &metrics, 0 );
 
     parent = GetAncestor( hwnd, GA_PARENT );
-    GetClientRect( parent, &rectParent );
+    if (parent == GetDesktopWindow())
+    {
+        MONITORINFO mon_info;
+        HMONITOR monitor = MonitorFromWindow( hwnd, MONITOR_DEFAULTTOPRIMARY );
+
+        mon_info.cbSize = sizeof( mon_info );
+        GetMonitorInfoW( monitor, &mon_info );
+        rectParent = mon_info.rcWork;
+    }
+    else GetClientRect( parent, &rectParent );
+
     if ((pt.x >= rectParent.left) && (pt.x + GetSystemMetrics(SM_CXICON) < rectParent.right) &&
         (pt.y >= rectParent.top) && (pt.y + GetSystemMetrics(SM_CYICON) < rectParent.bottom))
         return pt;  /* The icon already has a suitable position */
@@ -868,7 +882,7 @@ static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
 
     hrgn = CreateRectRgn( 0, 0, 0, 0 );
     tmp = CreateRectRgn( 0, 0, 0, 0 );
-    for (child = GetWindow( parent, GW_HWNDFIRST ); child; child = GetWindow( child, GW_HWNDNEXT ))
+    for (child = GetWindow( parent, GW_CHILD ); child; child = GetWindow( child, GW_HWNDNEXT ))
     {
         if (child == hwnd) continue;
         if ((GetWindowLongW( child, GWL_STYLE ) & (WS_VISIBLE|WS_MINIMIZE)) != (WS_VISIBLE|WS_MINIMIZE))
@@ -881,12 +895,30 @@ static POINT WINPOS_FindIconPos( HWND hwnd, POINT pt )
     }
     DeleteObject( tmp );
 
-    for (rect.bottom = rectParent.bottom; rect.bottom >= yspacing; rect.bottom -= yspacing)
+    for (y = 0; y < (rectParent.bottom - rectParent.top) / yspacing; y++)
     {
-        for (rect.left = rectParent.left; rect.left <= rectParent.right - xspacing; rect.left += xspacing)
+        if (metrics.iArrange & ARW_STARTTOP)
         {
-            rect.right = rect.left + xspacing;
+            rect.top = rectParent.top + y * yspacing;
+            rect.bottom = rect.top + yspacing;
+        }
+        else
+        {
+            rect.bottom = rectParent.bottom - y * yspacing;
             rect.top = rect.bottom - yspacing;
+        }
+        for (x = 0; x < (rectParent.right - rectParent.left) / xspacing; x++)
+        {
+            if (metrics.iArrange & ARW_STARTRIGHT)
+            {
+                rect.right = rectParent.right - x * xspacing;
+                rect.left = rect.right - xspacing;
+            }
+            else
+            {
+                rect.left = rectParent.left + x * xspacing;
+                rect.right = rect.left + xspacing;
+            }
             if (!RectInRegion( hrgn, &rect ))
             {
                 /* No window was found, so it's OK for us */
@@ -2348,7 +2380,11 @@ HDWP WINAPI DeferWindowPos( HDWP hdwp, HWND hwnd, HWND hwndAfter,
           hdwp, hwnd, hwndAfter, x, y, cx, cy, flags);
 
     hwnd = WIN_GetFullHandle( hwnd );
-    if (is_desktop_window( hwnd )) return 0;
+    if (is_desktop_window( hwnd ) || !IsWindow( hwnd ))
+    {
+        SetLastError( ERROR_INVALID_WINDOW_HANDLE );
+        return 0;
+    }
 
     if (!(pDWP = get_user_handle_ptr( hdwp, USER_DWP ))) return 0;
     if (pDWP == OBJ_OTHER_PROCESS)
@@ -2455,10 +2491,24 @@ UINT WINAPI ArrangeIconicWindows( HWND parent )
     RECT rectParent;
     HWND hwndChild;
     INT x, y, xspacing, yspacing;
+    POINT pt;
+    MINIMIZEDMETRICS metrics;
 
-    GetClientRect( parent, &rectParent );
-    x = rectParent.left;
-    y = rectParent.bottom;
+    metrics.cbSize = sizeof(metrics);
+    SystemParametersInfoW( SPI_GETMINIMIZEDMETRICS, sizeof(metrics), &metrics, 0 );
+
+    if (parent == GetDesktopWindow())
+    {
+        MONITORINFO mon_info;
+        HMONITOR monitor = MonitorFromWindow( 0, MONITOR_DEFAULTTOPRIMARY );
+
+        mon_info.cbSize = sizeof( mon_info );
+        GetMonitorInfoW( monitor, &mon_info );
+        rectParent = mon_info.rcWork;
+    }
+    else GetClientRect( parent, &rectParent );
+
+    x = y = 0;
     xspacing = GetSystemMetrics(SM_CXICONSPACING);
     yspacing = GetSystemMetrics(SM_CYICONSPACING);
 
@@ -2469,17 +2519,25 @@ UINT WINAPI ArrangeIconicWindows( HWND parent )
         {
             WINPOS_ShowIconTitle( hwndChild, FALSE );
 
-            SetWindowPos( hwndChild, 0, x + (xspacing - GetSystemMetrics(SM_CXICON)) / 2,
-                            y - yspacing - GetSystemMetrics(SM_CYICON)/2, 0, 0,
-                            SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
+            if (metrics.iArrange & ARW_STARTRIGHT)
+                pt.x = rectParent.right - (x + 1) * xspacing;
+            else
+                pt.x = rectParent.left + x * xspacing;
+            if (metrics.iArrange & ARW_STARTTOP)
+                pt.y = rectParent.top + y * yspacing;
+            else
+                pt.y = rectParent.bottom - (y + 1) * yspacing;
+
+            SetWindowPos( hwndChild, 0, pt.x + (xspacing - GetSystemMetrics(SM_CXICON)) / 2,
+                          pt.y + (yspacing - GetSystemMetrics(SM_CYICON)) / 2, 0, 0,
+                          SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
 	    if( IsWindow(hwndChild) )
                 WINPOS_ShowIconTitle(hwndChild , TRUE );
 
-            if (x <= rectParent.right - xspacing) x += xspacing;
-            else
+            if (++x >= (rectParent.right - rectParent.left) / xspacing)
             {
-                x = rectParent.left;
-                y -= yspacing;
+                x = 0;
+                y++;
             }
         }
         hwndChild = GetWindow( hwndChild, GW_HWNDNEXT );
