@@ -452,6 +452,11 @@ RPCRTAPI RPC_STATUS RPC_ENTRY
   RpcServerRegisterIf2( RPC_IF_HANDLE IfSpec, UUID* MgrTypeUuid, RPC_MGR_EPV* MgrEpv,
                         unsigned int Flags, unsigned int MaxCalls, unsigned int MaxRpcSize, RPC_IF_CALLBACK_FN* IfCallbackFn );
 
+RPCRTAPI RPC_STATUS RPC_ENTRY
+  RpcServerRegisterIf3( RPC_IF_HANDLE IfSpec, UUID* MgrTypeUuid, RPC_MGR_EPV* MgrEpv,
+                        unsigned int Flags, unsigned int MaxCalls, unsigned int MaxRpcSize,
+                        RPC_IF_CALLBACK_FN* IfCallbackFn, void* SecurityDescriptor );
+
 RPCRTAPI RPC_STATUS RPC_ENTRY
   RpcServerUnregisterIf( RPC_IF_HANDLE IfSpec, UUID* MgrTypeUuid, unsigned int WaitForCallsToComplete );
 
